package envoy.client.ui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.text.SimpleDateFormat;

import javax.swing.*;

import envoy.client.data.Settings;
import envoy.client.ui.list.ComponentList;
import envoy.client.ui.list.ComponentListCellRenderer;
import envoy.data.Message;

/**
 * Defines how a message is displayed.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>MessageListRenderer.java</strong><br>
 * Created: <strong>19 Oct 2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @author Leon Hofmeister
 * @since Envoy v0.1-alpha
 */
public class MessageListRenderer implements ComponentListCellRenderer<Message> {

	private JTextArea messageTextArea;

	@Override
	public JPanel getListCellComponent(ComponentList<? extends Message> list, Message value, boolean isSelected) {
		final JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		final Theme theme = Settings.getInstance().getThemes().get(Settings.getInstance().getCurrentTheme());

		// Panel background
		panel.setBackground(isSelected ? theme.getSelectionColor() : theme.getCellColor());

		// TODO: Handle message attachments

		final String	state	= value.getStatus().toString();
		final String	date	= new SimpleDateFormat("dd.MM.yyyy HH:mm").format(value.getCreationDate());
		final String	text	= value.getText();

		// The Label that displays the creation date of a message
		JLabel dateLabel = new JLabel(date);
		// Set the date color to be the value of DateColorChat
		dateLabel.setForeground(theme.getDateColor());

		panel.add(dateLabel, BorderLayout.NORTH);

		// The JTextArea that displays the text content of a message and its status
		messageTextArea = new JTextArea(text + System.getProperty("line.separator"));
		messageTextArea.setLineWrap(true);
		messageTextArea.setWrapStyleWord(true);
		messageTextArea.setAlignmentX(0.5f);
		messageTextArea.setForeground(theme.getMessageTextColor());
		messageTextArea.setBackground(panel.getBackground());
		messageTextArea.setEditable(false);

		panel.add(messageTextArea, BorderLayout.CENTER);

		JLabel statusLabel = new JLabel(state);
		statusLabel.setFont(new Font("Arial", Font.BOLD, 14));
		Color statusColor;
		switch (value.getStatus()) {
			case WAITING:
				statusColor = Color.gray;
				break;
			case SENT:
				statusColor = Color.blue;
				break;
			case RECEIVED:
				statusColor = Color.yellow;
				break;
			case READ:
				statusColor = Color.green;
				break;
			default:
				statusColor = theme.getMessageTextColor();
				break;
		}
		statusLabel.setForeground(statusColor);
		statusLabel.setBackground(panel.getBackground());

		panel.add(statusLabel, BorderLayout.SOUTH);

		// Define some space to the messages below
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(), BorderFactory.createEtchedBorder()));

		return panel;
	}
}
